import asyncio
import discord
from discord.ext import commands
import random as rd
import yaml

description = '''!help pour la liste des commandes'''
bot = commands.Bot(command_prefix='!', description=description)


class CharacterSheet:
	def __init__(self, userid):
		self._user_id = userid
		self._load()

	def __str__(self):
		d = {k:v for k,v in self.__dict__.items() if not k.startswith("_")}
		s = yaml.dump(d)
		pretty=""
		for line in s.splitlines():
			l = ""
			for c in line:
				if c != '"':
					l+=c
			pretty += l+"\n"
		return "```{}```".format(pretty)
	def _help(self):
		meth_list = ", ".join([func for func in dir(self) if callable(getattr(self, func)) and not func.startswith("_")])
		return "Commandes : "+meth_list
		
	def additem(self, item):
		if item in self.inventory.keys():
			self.inventory[item] += 1
		else:
			self.inventory[item] = 1
	def remitem(self, item):
		if item in self.inventory.keys() and self.inventory[item]>0:
			self.inventory[item] -= 1
		else:
			print("No {} to remove".format(item))
	def rollstats(self):
		import random as rd
		s = ["str", "int", "cha", "dex", "con", "wis"]
		self._bonus=[-3,-3,-3,-2,-2,-2,0,0,0,0,1,1,1,2,2,3]
		self.stats = {}
		for k in s:
			self.stats[k] = sum([rd.randint(1,6) for i in range(3)])
		self.gold = 10*sum([rd.randint(1,6) for i in range(3)])
		try:
			self._dv = {"voleur":4, "clerc":6, "guerrier":8, "mage":4}[self.classe.lower()]
			self._dv += {"humain":0, "halfelin":1, "nain":2, "elfe":2}[self.race.lower()]
		except KeyError:
			self._dv = 8
		self.AC = 9-self._bonus[self.stats["dex"]-3]
		self.HP = rd.randint(1,self._dv)+self._bonus[self.stats["con"]-3]
	def levelup(self):
		self.level += 1
		self.xp = 0
		self.HP += rd.randint(1,self._dv)+self._bonus[self.stats["con"]-3]

	def note(self, text):
		self.notes.append(text)
	def pay(self, amount):
		try:
			amount=int(amount)
		except ValueError:
			print("amount not castable as an integer.")
		self.gold -= amount
	def editattribute(self, attr, value):
		if hasattr(self, attr):
			self.__dict__[attr] = eval(value)
	def new(self, name, class_, race, alignment):
		self.name = name
		self.alignment = alignment
		self.xp = 0
		self.level = 1
		self.notes = []
		self.classe = class_
		self.race = race
		self.rollstats()
		self.inventory = {"sword":1, "leather armor":1}
	def _save(self):
		with open("characters/{}.yaml".format(self._user_id), "w") as f:
			f.write(yaml.dump(self.__dict__))
	def _load(self):
		try:
			loader = open("characters/{}.yaml".format(self._user_id), "r")
			self.__dict__ = yaml.load(loader)
		except FileNotFoundError:
			pass
		finally:
			pass
			#loader.close()

@bot.command(pass_context=True)
async def character(ctx, command=None, *args):
	"""Affiche des infos sur votre personnage, ou permet de les éditer."""
	cs = CharacterSheet(ctx.message.author.id)
	if command is not None:
		command = command.lower()
	if command is None or command == "info":
		await bot.say(str(cs))
	elif command == "help":
		await bot.say(cs._help())
	elif not command.startswith("_"):
		method = getattr(cs, command, None)
		if callable(method):
			method(*args)
	cs._save()


@bot.command(pass_context=True)
async def roll(ctx, dice):
	"""Lance un dé au format XdY. Utilisez le flag -v pour un résultat plus complet."""
	makesum = True
	if dice.startswith("d"):
		dice = "1" + dice
		makesum = False
	try:
		rolls, limit = map(int, dice.split('d'))
	except Exception:
		await bot.say('Le format est XdY !')
		return
	ro = [rd.randint(1, limit) for _ in range(rolls)]
	s = sum(ro)
	user = ctx.message.author.id
	if makesum:
		result = '+'.join(str(r) for r in ro)
		result = "<@!{0}> a obtenu {1} sur {2} ({3} = {1}).".format(user, s, dice, result)
	else:
		result = "<@!{}> a obtenu {} sur {}".format(user, s, dice)
	await bot.say(result)

bot.run("Mzc2NDU5NTUzNzc0MjM5NzU1.DN-tWQ.qByRdEkrLDcgWyspvF7FeJ3i-28")
